/* ------------------------------------------------------------------------- *
 * Copyright 2002-2023, OpenNebula Project, OpenNebula Systems               *
 *                                                                           *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may   *
 * not use this file except in compliance with the License. You may obtain   *
 * a copy of the License at                                                  *
 *                                                                           *
 * http://www.apache.org/licenses/LICENSE-2.0                                *
 *                                                                           *
 * Unless required by applicable law or agreed to in writing, software       *
 * distributed under the License is distributed on an "AS IS" BASIS,         *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  *
 * See the License for the specific language governing permissions and       *
 * limitations under the License.                                            *
 * ------------------------------------------------------------------------- */
/* eslint-disable jsdoc/require-jsdoc */
import PropTypes from 'prop-types'
import { Box } from '@mui/material'
import { useFormContext, useWatch } from 'react-hook-form'

import { AttributePanel } from 'client/components/Tabs/Common'
import { SCHEMA } from 'client/components/Forms/VmTemplate/CreateForm/Steps/CustomVariables/schema'
import { cleanEmpty, cloneObject, set } from 'client/utils'
import { T, ACTIONS } from 'client/constants'

const ALL_ACTIONS = [
  ACTIONS.COPY_ATTRIBUTE,
  ACTIONS.ADD_ATTRIBUTE,
  ACTIONS.EDIT_ATTRIBUTE,
  ACTIONS.DELETE_ATTRIBUTE,
]

export const STEP_ID = 'custom-variables'

const Content = () => {
  const { setValue } = useFormContext()
  const customVars = useWatch({ name: STEP_ID })

  const handleChangeAttribute = (path, newValue) => {
    const newCustomVars = cloneObject(customVars)

    set(newCustomVars, path, newValue)
    setValue(STEP_ID, cleanEmpty(newCustomVars))
  }

  return (
    <Box display="grid" gap="1em">
      <AttributePanel
        handleAdd={handleChangeAttribute}
        handleEdit={handleChangeAttribute}
        handleDelete={handleChangeAttribute}
        attributes={customVars}
        actions={ALL_ACTIONS}
        filtersSpecialAttributes={false}
      />
    </Box>
  )
}

const CustomVariables = () => ({
  id: STEP_ID,
  label: T.CustomVariables,
  resolver: SCHEMA,
  optionsValidate: { abortEarly: false },
  content: Content,
})

Content.propTypes = {
  data: PropTypes.any,
}

export default CustomVariables
